@@ -1,24 +1,24 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, weakref, operator
+import regex, weakref, operator
 from functools import partial
 from datetime import timedelta
 from collections import deque, OrderedDict
 
-from calibre.constants import preferred_encoding
+from calibre.constants import preferred_encoding, DEBUG
 from calibre.db.utils import force_to_bool
 from calibre.utils.config_base import prefs
 from calibre.utils.date import parse_date, UNDEFINED_DATE, now, dt_as_local
 from calibre.utils.icu import primary_contains, sort_key
 from calibre.utils.localization import lang_map, canonicalize_lang
 from calibre.utils.search_query_parser import SearchQueryParser, ParseException
+from polyglot.builtins import iteritems, unicode_type, string_or_bytes
 
 CONTAINS_MATCH = 0
 EQUALS_MATCH   = 1
@@ -26,6 +26,7 @@ REGEXP_MATCH   = 2
 
 # Utils {{{
 
+
 def _matchkind(query, case_sensitive=False):
     matchkind = CONTAINS_MATCH
     if (len(query) > 1):
@@ -43,6 +44,7 @@ def _matchkind(query, case_sensitive=False):
         query = icu_lower(query)
     return matchkind, query
 
+
 def _match(query, value, matchkind, use_primary_find_in_search=True, case_sensitive=False):
     if query.startswith('..'):
         query = query[1:]
@@ -70,19 +72,21 @@ def _match(query, value, matchkind, use_primary_find_in_search=True, case_sensit
                 elif query == t:
                     return True
             elif matchkind == REGEXP_MATCH:
-                if re.search(query, t, re.UNICODE if case_sensitive else re.I|re.UNICODE):
+                flags = regex.UNICODE | regex.VERSION1 | regex.FULLCASE | (0 if case_sensitive else regex.IGNORECASE)
+                if regex.search(query, t, flags) is not None:
                     return True
             elif matchkind == CONTAINS_MATCH:
                 if not case_sensitive and use_primary_find_in_search:
                     if primary_contains(query, t):
                         return True
                 elif query in t:
-                        return True
-        except re.error:
+                    return True
+        except regex.error:
             pass
     return False
 # }}}
 
+
 class DateSearch(object):  # {{{
 
     def __init__(self):
@@ -97,7 +101,7 @@ class DateSearch(object):  # {{{
         self.local_today         = {'_today', 'today', icu_lower(_('today'))}
         self.local_yesterday     = {'_yesterday', 'yesterday', icu_lower(_('yesterday'))}
         self.local_thismonth     = {'_thismonth', 'thismonth', icu_lower(_('thismonth'))}
-        self.daysago_pat = re.compile(r'(%s|daysago|_daysago)$'%_('daysago'))
+        self.daysago_pat = regex.compile(r'(%s|daysago|_daysago)$'%_('daysago'), flags=regex.UNICODE | regex.VERSION1)
 
     def eq(self, dbdate, query, field_count):
         if dbdate.year == query.year:
@@ -145,7 +149,9 @@ class DateSearch(object):  # {{{
 
         if query == 'false':
             for v, book_ids in field_iter():
-                if isinstance(v, (str, unicode)):
+                if isinstance(v, (bytes, unicode_type)):
+                    if isinstance(v, bytes):
+                        v = v.decode(preferred_encoding, 'replace')
                     v = parse_date(v)
                 if v is None or v <= UNDEFINED_DATE:
                     matches |= book_ids
@@ -153,13 +159,15 @@ class DateSearch(object):  # {{{
 
         if query == 'true':
             for v, book_ids in field_iter():
-                if isinstance(v, (str, unicode)):
+                if isinstance(v, (bytes, unicode_type)):
+                    if isinstance(v, bytes):
+                        v = v.decode(preferred_encoding, 'replace')
                     v = parse_date(v)
                 if v is not None and v > UNDEFINED_DATE:
                     matches |= book_ids
             return matches
 
-        for k, relop in self.operators.iteritems():
+        for k, relop in iteritems(self.operators):
             if query.startswith(k):
                 query = query[len(k):]
                 break
@@ -195,7 +203,7 @@ class DateSearch(object):  # {{{
                     field_count = query.count('/') + 1
 
         for v, book_ids in field_iter():
-            if isinstance(v, (str, unicode)):
+            if isinstance(v, string_or_bytes):
                 v = parse_date(v)
             if v is not None and relop(dt_as_local(v), qd, field_count):
                 matches |= book_ids
@@ -203,6 +211,7 @@ class DateSearch(object):  # {{{
         return matches
 # }}}
 
+
 class NumericSearch(object):  # {{{
 
     def __init__(self):
@@ -245,7 +254,7 @@ class NumericSearch(object):  # {{{
             else:
                 relop = lambda x,y: x is not None
         else:
-            for k, relop in self.operators.iteritems():
+            for k, relop in iteritems(self.operators):
                 if query.startswith(k):
                     query = query[len(k):]
                     break
@@ -256,7 +265,8 @@ class NumericSearch(object):  # {{{
                 cast = lambda x: 0 if x is None else int(x)
                 adjust = lambda x: x // 2
             else:
-                cast = float if dt in ('float', 'composite', 'half-rating') else int
+                # Datatype is empty if the source is a tempate. Assume float
+                cast = float if dt in ('float', 'composite', 'half-rating', '') else int
 
             mult = 1.0
             if len(query) > 1:
@@ -294,6 +304,7 @@ class NumericSearch(object):  # {{{
 
 # }}}
 
+
 class BooleanSearch(object):  # {{{
 
     def __init__(self):
@@ -335,6 +346,7 @@ class BooleanSearch(object):  # {{{
 
 # }}}
 
+
 class KeyPairSearch(object):  # {{{
 
     def __call__(self, query, field_iter, candidates, use_primary_find):
@@ -361,7 +373,7 @@ class KeyPairSearch(object):  # {{{
             return found if valq == 'true' else candidates - found
 
         for m, book_ids in field_iter():
-            for key, val in m.iteritems():
+            for key, val in iteritems(m):
                 if (keyq and not _match(keyq, (key,), keyq_mkind,
                                         use_primary_find_in_search=use_primary_find)):
                     continue
@@ -375,6 +387,7 @@ class KeyPairSearch(object):  # {{{
 
 # }}}
 
+
 class SavedSearchQueries(object):  # {{{
     queries = {}
     opt_name = ''
@@ -400,7 +413,7 @@ class SavedSearchQueries(object):  # {{{
         return self._db()
 
     def force_unicode(self, x):
-        if not isinstance(x, unicode):
+        if not isinstance(x, unicode_type):
             x = x.decode(preferred_encoding, 'replace')
         return x
 
@@ -433,9 +446,10 @@ class SavedSearchQueries(object):  # {{{
             db._set_pref(self.opt_name, smap)
 
     def names(self):
-        return sorted(self.queries.iterkeys(), key=sort_key)
+        return sorted(self.queries, key=sort_key)
 # }}}
 
+
 class Parser(SearchQueryParser):  # {{{
 
     def __init__(self, dbcache, all_book_ids, gst, date_search, num_search,
@@ -490,6 +504,15 @@ class Parser(SearchQueryParser):  # {{{
         if location not in self.all_search_locations:
             return matches
 
+        if location == 'vl':
+            vl = self.dbcache._pref('virtual_libraries', {}).get(query) if query else None
+            if not vl:
+                raise ParseException(_('No such Virtual library: {}').format(query))
+            try:
+                return candidates & self.dbcache.books_in_virtual_library(query)
+            except RuntimeError:
+                raise ParseException(_('Virtual library search is recursive: {}').format(query))
+
         if (len(location) > 2 and location.startswith('@') and
                     location[1:] in self.grouped_search_terms):
             location = location[1:]
@@ -565,6 +588,7 @@ class Parser(SearchQueryParser):  # {{{
                      fm['display'].get('composite_sort', '') == 'number')):
                 if location == 'id':
                     is_many = False
+
                     def fi(default_value=None):
                         for qid in candidates:
                             yield qid, {qid}
@@ -604,12 +628,53 @@ class Parser(SearchQueryParser):  # {{{
 
         # Everything else (and 'all' matches)
         case_sensitive = prefs['case_sensitive']
+
+        if location == 'template':
+            try:
+                template, sep, query = regex.split('#@#:([tdnb]):', query, flags=regex.IGNORECASE)
+                if sep:
+                    sep = sep.lower()
+                else:
+                    sep = 't'
+            except:
+                if DEBUG:
+                    import traceback
+                    traceback.print_exc()
+                raise ParseException(_('search template: missing or invalid separator. Valid separators are: {}').format('#@#:[tdnb]:'))
+            matchkind, query = _matchkind(query, case_sensitive=case_sensitive)
+            matches = set()
+            error_string = '*@*TEMPLATE_ERROR*@*'
+            template_cache = {}
+            for book_id in candidates:
+                mi = self.dbcache.get_proxy_metadata(book_id)
+                val = mi.formatter.safe_format(template, {}, error_string, mi,
+                                            column_name='search template',
+                                            template_cache=template_cache)
+                if val.startswith(error_string):
+                    raise ParseException(val[len(error_string):])
+                if sep == 't':
+                    if _match(query, [val,], matchkind, use_primary_find_in_search=upf,
+                              case_sensitive=case_sensitive):
+                        matches.add(book_id)
+                elif sep == 'n' and val:
+                    matches.update(self.num_search(
+                        icu_lower(query), {val:{book_id,}}.items, '', '',
+                        {book_id,}, is_many=False))
+                elif sep == 'd' and val:
+                    matches.update(self.date_search(
+                            icu_lower(query), {val:{book_id,}}.items))
+                elif sep == 'b':
+                    matches.update(self.bool_search(icu_lower(query),
+                            {'True' if val else 'False':{book_id,}}.items, False))
+
+            return matches
+
         matchkind, query = _matchkind(query, case_sensitive=case_sensitive)
         all_locs = set()
         text_fields = set()
         field_metadata = {}
 
-        for x, fm in self.field_metadata.iteritems():
+        for x, fm in self.field_metadata.iter_items():
             if x.startswith('@'):
                 continue
             if fm['search_terms'] and x not in {'series_sort', 'id'}:
@@ -647,7 +712,7 @@ class Parser(SearchQueryParser):  # {{{
                 q = canonicalize_lang(query)
                 if q is None:
                     lm = lang_map()
-                    rm = {v.lower():k for k,v in lm.iteritems()}
+                    rm = {v.lower():k for k,v in iteritems(lm)}
                     q = rm.get(query, query)
 
             if matchkind == CONTAINS_MATCH and q.lower() in {'true', 'false'}:
@@ -683,7 +748,7 @@ class Parser(SearchQueryParser):  # {{{
             if location in text_fields:
                 for val, book_ids in self.field_iter(location, current_candidates):
                     if val is not None:
-                        if isinstance(val, basestring):
+                        if isinstance(val, string_or_bytes):
                             val = (val,)
                         if _match(q, val, matchkind, use_primary_find_in_search=upf, case_sensitive=case_sensitive):
                             matches |= book_ids
@@ -722,6 +787,7 @@ class Parser(SearchQueryParser):  # {{{
         return matches
 # }}}
 
+
 class LRUCache(object):  # {{{
 
     'A simple Least-Recently-Used cache'
@@ -775,9 +841,10 @@ class LRUCache(object):  # {{{
         return self.get(key)
 
     def __iter__(self):
-        return self.item_map.iteritems()
+        return iteritems(self.item_map)
 # }}}
 
+
 class Search(object):
 
     MAX_CACHE_UPDATE = 50
@@ -860,6 +927,20 @@ class Search(object):
         finally:
             sqp.dbcache = sqp.lookup_saved_search = None
 
+    def query_is_cacheable(self, sqp, dbcache, query):
+        if query:
+            for name, value in sqp.get_queried_fields(query):
+                if name == 'template' and '#@#:d:' in value:
+                    return False
+                elif name in dbcache.field_metadata.all_field_keys():
+                    fm = dbcache.field_metadata[name]
+                    if fm['datatype'] == 'datetime':
+                        return False
+                    if fm['datatype'] == 'composite':
+                        if fm.get('display', {}).get('composite_sort', '') == 'date':
+                            return False
+        return True
+
     def _do_search(self, sqp, query, search_restriction, dbcache, book_ids=None):
         ''' Do the search, caching the results. Results are cached only if the
         search is on the full library and no virtual field is searched on '''
@@ -869,30 +950,36 @@ class Search(object):
             query = query.decode('utf-8')
 
         query = query.strip()
-        if book_ids is None and query and not search_restriction:
+        use_cache = self.query_is_cacheable(sqp, dbcache, query)
+
+        if use_cache and book_ids is None and query and not search_restriction:
             cached = self.cache.get(query)
             if cached is not None:
                 return cached
 
         restricted_ids = all_book_ids = dbcache._all_book_ids(type=set)
         if search_restriction and search_restriction.strip():
-            cached = self.cache.get(search_restriction.strip())
-            if cached is None:
-                sqp.all_book_ids = all_book_ids if book_ids is None else book_ids
-                restricted_ids = sqp.parse(search_restriction)
-                if not sqp.virtual_field_used and sqp.all_book_ids is all_book_ids:
-                    self.cache.add(search_restriction.strip(), restricted_ids)
+            sr = search_restriction.strip()
+            sqp.all_book_ids = all_book_ids if book_ids is None else book_ids
+            if self.query_is_cacheable(sqp, dbcache, sr):
+                cached = self.cache.get(sr)
+                if cached is None:
+                    restricted_ids = sqp.parse(sr)
+                    if not sqp.virtual_field_used and sqp.all_book_ids is all_book_ids:
+                        self.cache.add(sr, restricted_ids)
+                else:
+                    restricted_ids = cached
+                    if book_ids is not None:
+                        restricted_ids = book_ids.intersection(restricted_ids)
             else:
-                restricted_ids = cached
-                if book_ids is not None:
-                    restricted_ids = book_ids.intersection(restricted_ids)
+                restricted_ids = sqp.parse(sr)
         elif book_ids is not None:
             restricted_ids = book_ids
 
         if not query:
             return restricted_ids
 
-        if restricted_ids is all_book_ids:
+        if use_cache and restricted_ids is all_book_ids:
             cached = self.cache.get(query)
             if cached is not None:
                 return cached
@@ -904,4 +991,3 @@ class Search(object):
             self.cache.add(query, result)
 
         return result
-
