@@ -18,7 +18,7 @@ from PyQt5.Qt import (
     QDesktopServices, QDialog, QEvent, QFileDialog, QFileIconProvider, QFileInfo, QPalette,
     QFont, QFontDatabase, QFontInfo, QFontMetrics, QIcon, QLocale, QColor,
     QNetworkProxyFactory, QObject, QSettings, QSocketNotifier, QStringListModel, Qt,
-    QThread, QTimer, QTranslator, QUrl, pyqtSignal
+    QThread, QTimer, QTranslator, QUrl, pyqtSignal, QIODevice, QDialogButtonBox
 )
 from PyQt5.QtWidgets import QStyle  # Gives a nicer error message than import from Qt
 
@@ -44,7 +44,7 @@ from polyglot.builtins import (
 )
 
 try:
-    NO_URL_FORMATTING = QUrl.None_
+    NO_URL_FORMATTING = QUrl.UrlFormattingOption.None_
 except AttributeError:
     NO_URL_FORMATTING = getattr(QUrl, 'None')
 
@@ -117,7 +117,8 @@ def create_defs():
 
     defs['action-layout-context-menu-device'] = (
             'View', 'Save To Disk', None, 'Remove Books', None,
-            'Add To Library', 'Edit Collections', 'Match Books'
+            'Add To Library', 'Edit Collections', 'Match Books',
+            'Show Matched Book In Library'
             )
 
     defs['action-layout-context-menu-cover-browser'] = (
@@ -317,9 +318,9 @@ config = _config()
 
 # }}}
 
-QSettings.setPath(QSettings.IniFormat, QSettings.UserScope, config_dir)
-QSettings.setPath(QSettings.IniFormat, QSettings.SystemScope, config_dir)
-QSettings.setDefaultFormat(QSettings.IniFormat)
+QSettings.setPath(QSettings.Format.IniFormat, QSettings.Scope.UserScope, config_dir)
+QSettings.setPath(QSettings.Format.IniFormat, QSettings.Scope.SystemScope, config_dir)
+QSettings.setDefaultFormat(QSettings.Format.IniFormat)
 
 
 def default_author_link():
@@ -434,7 +435,7 @@ def question_dialog(parent, title, msg, det_msg='', show_copy_button=False,
         tc.setChecked(bool(skip_dialog_skip_precheck))
         d.resize_needed.emit()
 
-    ret = d.exec_() == d.Accepted
+    ret = d.exec_() == QDialog.DialogCode.Accepted
 
     if skip_dialog_name is not None and not d.toggle_checkbox.isChecked():
         auto_skip.add(skip_dialog_name)
@@ -457,7 +458,7 @@ def info_dialog(parent, title, msg, det_msg='', show=False,
 def show_restart_warning(msg, parent=None):
     d = warning_dialog(parent, _('Restart needed'), msg,
             show_copy_button=False)
-    b = d.bb.addButton(_('&Restart calibre now'), d.bb.AcceptRole)
+    b = d.bb.addButton(_('&Restart calibre now'), QDialogButtonBox.ButtonRole.AcceptRole)
     b.setIcon(QIcon(I('lt.png')))
     d.do_restart = False
 
@@ -484,9 +485,9 @@ class Dispatcher(QObject):
     def __init__(self, func, queued=True, parent=None):
         QObject.__init__(self, parent)
         self.func = func
-        typ = Qt.QueuedConnection
+        typ = Qt.ConnectionType.QueuedConnection
         if not queued:
-            typ = Qt.AutoConnection if queued is None else Qt.DirectConnection
+            typ = Qt.ConnectionType.AutoConnection if queued is None else Qt.ConnectionType.DirectConnection
         self.dispatch_signal.connect(self.dispatch, type=typ)
 
     def __call__(self, *args, **kwargs):
@@ -516,9 +517,9 @@ class FunctionDispatcher(QObject):
 
         QObject.__init__(self, parent)
         self.func = func
-        typ = Qt.QueuedConnection
+        typ = Qt.ConnectionType.QueuedConnection
         if not queued:
-            typ = Qt.AutoConnection if queued is None else Qt.DirectConnection
+            typ = Qt.ConnectionType.AutoConnection if queued is None else Qt.ConnectionType.DirectConnection
         self.dispatch_signal.connect(self.dispatch, type=typ)
         self.q = queue.Queue()
         self.lock = threading.Lock()
@@ -552,8 +553,8 @@ class GetMetadata(QObject):
 
     def __init__(self):
         QObject.__init__(self)
-        self.edispatch.connect(self._get_metadata, type=Qt.QueuedConnection)
-        self.idispatch.connect(self._from_formats, type=Qt.QueuedConnection)
+        self.edispatch.connect(self._get_metadata, type=Qt.ConnectionType.QueuedConnection)
+        self.idispatch.connect(self._from_formats, type=Qt.ConnectionType.QueuedConnection)
 
     def __call__(self, id, *args, **kwargs):
         self.edispatch.emit(id, args, kwargs)
@@ -638,9 +639,9 @@ class FileIconProvider(QFileIconProvider):
     def icon(self, arg):
         if isinstance(arg, QFileInfo):
             return self.load_icon(arg)
-        if arg == QFileIconProvider.Folder:
+        if arg == QFileIconProvider.IconType.Folder:
             return self.icons['dir']
-        if arg == QFileIconProvider.File:
+        if arg == QFileIconProvider.IconType.File:
             return self.icons['default']
         return QFileIconProvider.icon(self, arg)
 
@@ -680,12 +681,12 @@ else:
 
 def is_dark_theme():
     pal = QApplication.instance().palette()
-    col = pal.color(pal.Window)
+    col = pal.color(QPalette.ColorRole.Window)
     return max(col.getRgb()[:3]) < 115
 
 
 def choose_osx_app(window, name, title, default_dir='/Applications'):
-    fd = FileDialog(title=title, parent=window, name=name, mode=QFileDialog.ExistingFile,
+    fd = FileDialog(title=title, parent=window, name=name, mode=QFileDialog.FileMode.ExistingFile,
             default_dir=default_dir)
     app = fd.get_files()
     fd.setParent(None)
@@ -708,7 +709,7 @@ def pixmap_to_data(pixmap, format='JPEG', quality=None):
             quality = 90
     ba = QByteArray()
     buf = QBuffer(ba)
-    buf.open(QBuffer.WriteOnly)
+    buf.open(QIODevice.OpenModeFlag.WriteOnly)
     pixmap.save(buf, format, quality=quality)
     return ba.data()
 
@@ -820,11 +821,11 @@ def setup_hidpi():
     if hidpi == 'on' or (hidpi == 'auto' and not has_env_setting):
         if DEBUG:
             prints('Turning on automatic hidpi scaling')
-        QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True)
+        QApplication.setAttribute(Qt.ApplicationAttribute.AA_EnableHighDpiScaling, True)
     elif hidpi == 'off':
         if DEBUG:
             prints('Turning off automatic hidpi scaling')
-        QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, False)
+        QApplication.setAttribute(Qt.ApplicationAttribute.AA_EnableHighDpiScaling, False)
         for p in env_vars:
             os.environ.pop(p, None)
     elif DEBUG:
@@ -851,9 +852,9 @@ def setup_unix_signals(self):
         original_handlers[sig] = signal.signal(sig, lambda x, y: None)
         signal.siginterrupt(sig, False)
     signal.set_wakeup_fd(write_fd)
-    self.signal_notifier = QSocketNotifier(read_fd, QSocketNotifier.Read, self)
+    self.signal_notifier = QSocketNotifier(read_fd, QSocketNotifier.Type.Read, self)
     self.signal_notifier.setEnabled(True)
-    self.signal_notifier.activated.connect(self.signal_received, type=Qt.QueuedConnection)
+    self.signal_notifier.activated.connect(self.signal_received, type=Qt.ConnectionType.QueuedConnection)
     return original_handlers
 
 
@@ -893,7 +894,7 @@ class Application(QApplication):
         QApplication.setApplicationName(APP_UID)
         if override_program_name and hasattr(QApplication, 'setDesktopFileName'):
             QApplication.setDesktopFileName(override_program_name)
-        QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)  # needed for webengine
+        QApplication.setAttribute(Qt.ApplicationAttribute.AA_ShareOpenGLContexts, True)  # needed for webengine
         QApplication.__init__(self, qargs)
         sh = self.styleHints()
         if hasattr(sh, 'setShowShortcutsInContextMenus'):
@@ -901,8 +902,8 @@ class Application(QApplication):
         if ismacos:
             from calibre_extensions.cocoa import disable_cocoa_ui_elements
             disable_cocoa_ui_elements()
-        self.setAttribute(Qt.AA_UseHighDpiPixmaps)
-        self.setAttribute(Qt.AA_SynthesizeTouchForUnhandledMouseEvents, False)
+        self.setAttribute(Qt.ApplicationAttribute.AA_UseHighDpiPixmaps)
+        self.setAttribute(Qt.ApplicationAttribute.AA_SynthesizeTouchForUnhandledMouseEvents, False)
         try:
             base_dir()
         except EnvironmentError as err:
@@ -918,7 +919,7 @@ class Application(QApplication):
         if not iswindows:
             self.setup_unix_signals()
         if islinux or isbsd:
-            self.setAttribute(Qt.AA_DontUseNativeMenuBar, 'CALIBRE_NO_NATIVE_MENUBAR' in os.environ)
+            self.setAttribute(Qt.ApplicationAttribute.AA_DontUseNativeMenuBar, 'CALIBRE_NO_NATIVE_MENUBAR' in os.environ)
         self.setup_styles(force_calibre_style)
         self.setup_ui_font()
         if not self.using_calibre_style and self.style().objectName() == 'fusion':
@@ -969,8 +970,8 @@ class Application(QApplication):
             # same as non highlighted colors. This is a regression from Qt 4.
             # https://bugreports.qt-project.org/browse/QTBUG-41060
             p = self.palette()
-            for role in (p.Highlight, p.HighlightedText, p.Base, p.AlternateBase):
-                p.setColor(p.Inactive, role, p.color(p.Active, role))
+            for role in (QPalette.ColorRole.Highlight, QPalette.ColorRole.HighlightedText, QPalette.ColorRole.Base, QPalette.ColorRole.AlternateBase):
+                p.setColor(QPalette.ColorGroup.Inactive, role, p.color(QPalette.ColorGroup.Active, role))
             self.setPalette(p)
 
             # Prevent text copied to the clipboard from being lost on quit due to
@@ -1067,7 +1068,7 @@ class Application(QApplication):
         # Workaround for https://bugreports.qt.io/browse/QTBUG-75321
         # Buttontext is set to black for some reason
         pal = QPalette(self.palette())
-        pal.setColor(pal.ButtonText, pal.color(pal.WindowText))
+        pal.setColor(QPalette.ColorRole.ButtonText, pal.color(QPalette.ColorRole.WindowText))
         self.ignore_palette_changes = True
         self.setPalette(pal, 'QComboBox')
         self.ignore_palette_changes = False
@@ -1079,7 +1080,7 @@ class Application(QApplication):
         # appearance is changed. And it has to be after current event
         # processing finishes as of Qt 5.14 otherwise the palette change is
         # ignored.
-        QTimer.singleShot(1000, lambda: QApplication.instance().setAttribute(Qt.AA_SetPalette, False))
+        QTimer.singleShot(1000, lambda: QApplication.instance().setAttribute(Qt.ApplicationAttribute.AA_SetPalette, False))
         self.ignore_palette_changes = False
 
     def on_palette_change(self):
@@ -1135,8 +1136,8 @@ class Application(QApplication):
         if ismacos:
             from calibre_extensions.cocoa import transient_scroller
             transient_scroller = transient_scroller()
-        icon_map[QStyle.SP_CustomBase + 1] = I('close-for-light-theme.png')
-        icon_map[QStyle.SP_CustomBase + 2] = I('close-for-dark-theme.png')
+        icon_map[QStyle.StandardPixmap.SP_CustomBase + 1] = I('close-for-light-theme.png')
+        icon_map[QStyle.StandardPixmap.SP_CustomBase + 2] = I('close-for-dark-theme.png')
         self.pi.load_style(icon_map, transient_scroller)
 
     def _send_file_open_events(self):
@@ -1152,7 +1153,13 @@ class Application(QApplication):
         self.installTranslator(self._translator)
 
     def event(self, e):
-        if callable(self.file_event_hook) and e.type() == QEvent.FileOpen:
+        if callable(self.file_event_hook) and e.type() == QEvent.Type.FileOpen:
+            url = e.url().toString(QUrl.ComponentFormattingOption.FullyEncoded)
+            if url and url.startswith('calibre://'):
+                with self._file_open_lock:
+                    self._file_open_paths.append(url)
+                QTimer.singleShot(1000, self._send_file_open_events)
+                return True
             path = unicode_type(e.file())
             if os.access(path, os.R_OK):
                 with self._file_open_lock:
@@ -1307,7 +1314,7 @@ def ensure_app(headless=True):
                 if ismacos:
                     os.environ['QT_MAC_DISABLE_FOREGROUND_APPLICATION_TRANSFORM'] = '1'
             if headless and iswindows:
-                QApplication.setAttribute(Qt.AA_UseSoftwareOpenGL, True)
+                QApplication.setAttribute(Qt.ApplicationAttribute.AA_UseSoftwareOpenGL, True)
             _store_app = QApplication(args)
             if headless and has_headless:
                 _store_app.headless = True
@@ -1497,15 +1504,15 @@ def add_to_recent_docs(path):
 
 
 def windows_is_system_dark_mode_enabled():
-    s = QSettings(r"HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Themes\Personalize", QSettings.NativeFormat)
-    if s.status() == QSettings.NoError:
+    s = QSettings(r"HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Themes\Personalize", QSettings.Format.NativeFormat)
+    if s.status() == QSettings.Status.NoError:
         return s.value("AppsUseLightTheme") == 0
     return False
 
 
 def make_view_use_window_background(view):
     p = view.palette()
-    p.setColor(p.Base, p.color(p.Window))
-    p.setColor(p.AlternateBase, p.color(p.Window))
+    p.setColor(QPalette.ColorRole.Base, p.color(QPalette.ColorRole.Window))
+    p.setColor(QPalette.ColorRole.AlternateBase, p.color(QPalette.ColorRole.Window))
     view.setPalette(p)
     return view
