@@ -126,6 +126,7 @@ class FormatterFunction(object):
     category = 'Unknown'
     arg_count = 0
     aliases = []
+    is_python = True
 
     def evaluate(self, formatter, kwargs, mi, locals, *args):
         raise NotImplementedError()
@@ -145,7 +146,7 @@ class BuiltinFormatterFunction(FormatterFunction):
     def __init__(self):
         formatter_functions().register_builtin(self)
         eval_func = inspect.getmembers(self.__class__,
-                        lambda x: inspect.ismethod(x) and x.__name__ == 'evaluate')
+                        lambda x: inspect.isfunction(x) and x.__name__ == 'evaluate')
         try:
             lines = [l[4:] for l in inspect.getsourcelines(eval_func[0][1])[0]]
         except:
@@ -403,6 +404,26 @@ class BuiltinAssign(BuiltinFormatterFunction):
         return value
 
 
+class BuiltinListSplit(BuiltinFormatterFunction):
+    name = 'list_split'
+    arg_count = 3
+    category = 'List manipulation'
+    __doc__ = doc = _('split(list_val, sep, id_prefix) -- splits the list_val '
+                    "into separate values using 'sep', then assigns the values "
+                    "to variables named 'id_prefix_N' where N is the position "
+                    "of the value in the list. The first item has position 0 (zero). "
+                    "The function returns the last element in the list. "
+                    "Example: split('one, two, foo', ',', 'var') is equivalent "
+                    "to var_0 = 'one'; var_1 = 'two'; var_3 = 'foo'.")
+
+    def evaluate(self, formatter, kwargs, mi, locals, list_val, sep, id_prefix):
+        l = [v.strip() for v in list_val.split(sep)]
+        res = ''
+        for i,v in enumerate(l):
+            res = locals[id_prefix+'_'+unicode_type(i)] = v
+        return res
+
+
 class BuiltinPrint(BuiltinFormatterFunction):
     name = 'print'
     arg_count = -1
@@ -1180,6 +1201,26 @@ class BuiltinOndevice(BuiltinFormatterFunction):
         return _('This function can be used only in the GUI')
 
 
+class BuiltinAnnotationCount(BuiltinFormatterFunction):
+    name = 'annotation_count'
+    arg_count = 0
+    category = 'Get values from metadata'
+    __doc__ = doc = _('annotation_count() -- return the total number of annotations '
+                      'of all types attached to the current book. '
+                      'This function works only in the GUI.')
+
+    def evaluate(self, formatter, kwargs, mi, locals):
+        if hasattr(mi, '_proxy_metadata'):
+            try:
+                from calibre.gui2.ui import get_gui
+                c = get_gui().current_db.new_api.annotation_count_for_book(mi.id)
+                return '' if c == 0 else unicode_type(c)
+            except:
+                return _('Failed to fetch annotation count')
+            return ''
+        return _('This function can be used only in the GUI')
+
+
 class BuiltinSeriesSort(BuiltinFormatterFunction):
     name = 'series_sort'
     arg_count = 0
@@ -1285,16 +1326,9 @@ class BuiltinListUnion(BuiltinFormatterFunction):
     aliases = ['merge_lists']
 
     def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
-        res = [l.strip() for l in list1.split(separator) if l.strip()]
-        l2 = [l.strip() for l in list2.split(separator) if l.strip()]
-        lcl1 = {icu_lower(l) for l in res}
-
-        for i in l2:
-            if icu_lower(i) not in lcl1 and i not in res:
-                res.append(i)
-        if separator == ',':
-            return ', '.join(res)
-        return separator.join(res)
+        res = {icu_lower(l.strip()): l.strip() for l in list2.split(separator) if l.strip()}
+        res.update({icu_lower(l.strip()): l.strip() for l in list1.split(separator) if l.strip()})
+        return separator.join(res.values())
 
 
 class BuiltinListDifference(BuiltinFormatterFunction):
@@ -1701,6 +1735,40 @@ class BuiltinConnectedDeviceName(BuiltinFormatterFunction):
         return _('This function can be used only in the GUI')
 
 
+class BuiltinConnectedDeviceUUID(BuiltinFormatterFunction):
+    name = 'connected_device_uuid'
+    arg_count = 1
+    category = 'Get values from metadata'
+    __doc__ = doc = _("connected_device_uuid(storage_location) -- if a device is "
+                      "connected then return the device uuid (unique id), "
+                      "otherwise return the empty string. Each storage location "
+                      "on a device has a different uuid. The location names are "
+                      "'main', 'carda' and 'cardb'. This function works only in "
+                      "the GUI.")
+
+    def evaluate(self, formatter, kwargs, mi, locals, storage_location):
+        if hasattr(mi, '_proxy_metadata'):
+            # Do the import here so that we don't entangle the GUI when using
+            # command line functions
+            from calibre.gui2.ui import get_gui
+            info = get_gui().device_manager.get_current_device_information()
+            if info is None:
+                return ''
+            try:
+                if storage_location not in {'main', 'carda', 'cardb'}:
+                    raise ValueError(
+                         _('connected_device_name: invalid storage location "{0}"'
+                                    .format(storage_location)))
+                info = info['info'][4]
+                if storage_location not in info:
+                    return ''
+                return info[storage_location]['device_store_uuid']
+            except:
+                traceback.print_exc()
+                raise
+        return _('This function can be used only in the GUI')
+
+
 class BuiltinCheckYesNo(BuiltinFormatterFunction):
     name = 'check_yes_no'
     arg_count = 4
@@ -1783,21 +1851,62 @@ class BuiltinSwapAroundArticles(BuiltinFormatterFunction):
         return separator.join(sorted(result, key=sort_key))
 
 
+class BuiltinArguments(BuiltinFormatterFunction):
+    name = 'arguments'
+    arg_count = -1
+    category = 'other'
+    __doc__ = doc = _('arguments(id[=expression] [, id[=expression]]*) '
+                      '-- Used in a stored template to retrieve the arguments '
+                      'passed in the call. It both declares and initializes '
+                      'local variables, effectively parameters. The variables '
+                      'are positional; they get the value of the value given '
+                      'in the call in the same position. If the corresponding '
+                      'parameter is not provided in the call then arguments '
+                      'assigns that variable the provided default value. If '
+                      'there is no default value then the variable is set to '
+                      'the empty string.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        # The arguments function is implemented in-line in the formatter
+        raise NotImplementedError()
+
+
+class BuiltinGlobals(BuiltinFormatterFunction):
+    name = 'globals'
+    arg_count = -1
+    category = 'other'
+    __doc__ = doc = _('globals(id[=expression] [, id[=expression]]*) '
+                      '-- Retrieves "global variables" that can be passed into '
+                      'the formatter. It both declares and initializes local '
+                      'variables with the names of the global variables passed '
+                      'in. If the corresponding variable is not provided in '
+                      'the passed-in globals then it assigns that variable the '
+                      'provided default value. If there is no default value '
+                      'then the variable is set to the empty string.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        # The globals function is implemented in-line in the formatter
+        raise NotImplementedError()
+
+
 _formatter_builtins = [
-    BuiltinAdd(), BuiltinAnd(), BuiltinApproximateFormats(), BuiltinAssign(),
+    BuiltinAdd(), BuiltinAnd(), BuiltinApproximateFormats(), BuiltinArguments(),
+    BuiltinAssign(),
     BuiltinAuthorLinks(), BuiltinAuthorSorts(), BuiltinBooksize(),
     BuiltinCapitalize(), BuiltinCheckYesNo(), BuiltinCeiling(),
-    BuiltinCmp(), BuiltinConnectedDeviceName(), BuiltinContains(),
+    BuiltinCmp(), BuiltinConnectedDeviceName(), BuiltinConnectedDeviceUUID(), BuiltinContains(),
     BuiltinCount(), BuiltinCurrentLibraryName(), BuiltinCurrentLibraryPath(),
     BuiltinDaysBetween(), BuiltinDivide(), BuiltinEval(), BuiltinFirstNonEmpty(),
     BuiltinField(), BuiltinFinishFormatting(), BuiltinFirstMatchingCmp(), BuiltinFloor(),
     BuiltinFormatDate(), BuiltinFormatNumber(), BuiltinFormatsModtimes(),
     BuiltinFormatsPaths(), BuiltinFormatsSizes(), BuiltinFractionalPart(),
+    BuiltinGlobals(),
     BuiltinHasCover(), BuiltinHumanReadable(), BuiltinIdentifierInList(),
     BuiltinIfempty(), BuiltinLanguageCodes(), BuiltinLanguageStrings(),
     BuiltinInList(), BuiltinListDifference(), BuiltinListEquals(),
     BuiltinListIntersection(), BuiltinListitem(), BuiltinListRe(),
-    BuiltinListReGroup(), BuiltinListSort(), BuiltinListUnion(), BuiltinLookup(),
+    BuiltinListReGroup(), BuiltinListSort(), BuiltinListSplit(), BuiltinListUnion(),
+    BuiltinLookup(),
     BuiltinLowercase(), BuiltinMod(), BuiltinMultiply(), BuiltinNot(), BuiltinOndevice(),
     BuiltinOr(), BuiltinPrint(), BuiltinRatingToStars(), BuiltinRawField(), BuiltinRawList(),
     BuiltinRe(), BuiltinReGroup(), BuiltinRound(), BuiltinSelect(), BuiltinSeriesSort(),
@@ -1807,23 +1916,45 @@ _formatter_builtins = [
     BuiltinSwapAroundComma(), BuiltinSwitch(),
     BuiltinTemplate(), BuiltinTest(), BuiltinTitlecase(),
     BuiltinToday(), BuiltinTransliterate(), BuiltinUppercase(),
-    BuiltinUserCategories(), BuiltinVirtualLibraries()
+    BuiltinUserCategories(), BuiltinVirtualLibraries(), BuiltinAnnotationCount()
 ]
 
 
 class FormatterUserFunction(FormatterFunction):
 
-    def __init__(self, name, doc, arg_count, program_text):
+    def __init__(self, name, doc, arg_count, program_text, is_python):
+        self.is_python = is_python
         self.name = name
         self.doc = doc
         self.arg_count = arg_count
         self.program_text = program_text
+        self.cached_parse_tree = None
+
+    def to_pref(self):
+        return [self.name, self.doc, self.arg_count, self.program_text]
 
 
 tabs = re.compile(r'^\t*')
 
 
+def function_pref_is_python(pref):
+    if isinstance(pref, list):
+        pref = pref[3]
+    if pref.startswith('def'):
+        return True
+    if pref.startswith('program'):
+        return False
+    raise ValueError('Unknown program type in formatter function pref')
+
+
+def function_pref_name(pref):
+    return pref[0]
+
+
 def compile_user_function(name, doc, arg_count, eval_func):
+    if not function_pref_is_python(eval_func):
+        return FormatterUserFunction(name, doc, arg_count, eval_func, False)
+
     def replace_func(mo):
         return mo.group().replace('\t', '    ')
 
@@ -1838,7 +1969,7 @@ class UserFunction(FormatterUserFunction):
     if DEBUG and tweaks.get('enable_template_debug_printing', False):
         print(prog)
     exec(prog, locals_)
-    cls = locals_['UserFunction'](name, doc, arg_count, eval_func)
+    cls = locals_['UserFunction'](name, doc, arg_count, eval_func, True)
     return cls
 
 
@@ -1855,6 +1986,7 @@ def compile_user_template_functions(funcs):
             # then white space differences don't cause them to compare differently
 
             cls = compile_user_function(*func)
+            cls.is_python = function_pref_is_python(func)
             compiled_funcs[cls.name] = cls
         except Exception:
             try:
@@ -1862,7 +1994,7 @@ def compile_user_template_functions(funcs):
             except Exception:
                 func_name = 'Unknown'
             prints('**** Compilation errors in user template function "%s" ****' % func_name)
-            traceback.print_exc(limit=0)
+            traceback.print_exc(limit=10)
             prints('**** End compilation errors in %s "****"' % func_name)
     return compiled_funcs
 
